@@ -1,22 +1,37 @@
 /*
     This file is part of the KContacts framework.
-    SPDX-FileCopyrightText: 2016-2019 Laurent Montel <montel@kde.org>
+    Copyright (c) 2016 Laurent Montel <montel@kde.org>
 
-    SPDX-License-Identifier: LGPL-2.0-or-later
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either
+    version 2 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Library General Public License for more details.
+
+    You should have received a copy of the GNU Library General Public License
+    along with this library; see the file COPYING.LIB.  If not, write to
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include "orgtest.h"
 #include "org.h"
-#include "vcardtool_p.h"
-#include <QTest>
+#include "vcardtool.h"
+#include <qtest.h>
 
 OrgTest::OrgTest(QObject *parent)
     : QObject(parent)
 {
+
 }
 
 OrgTest::~OrgTest()
 {
+
 }
 
 void OrgTest::shouldHaveDefaultValue()
@@ -40,6 +55,7 @@ void OrgTest::shouldAssignValue()
     QCOMPARE(org.organization(), organization);
     QVERIFY(!org.parameters().isEmpty());
     QCOMPARE(org.parameters(), params);
+
 }
 
 void OrgTest::shouldAssignExternal()
@@ -70,6 +86,7 @@ void OrgTest::shouldSerialized()
     t >> result;
 
     QVERIFY(org == result);
+
 }
 
 void OrgTest::shouldEqualOrg()
@@ -103,7 +120,6 @@ void OrgTest::shouldParseOrg()
     QCOMPARE(lst.count(), 1);
     QCOMPARE(lst.at(0).extraOrganizationList().count(), 1);
     QCOMPARE(lst.at(0).organization(), QStringLiteral("boo"));
-    QCOMPARE(lst.at(0).extraOrganizationList().at(0).organization(), QStringLiteral("boo"));
 }
 
 void OrgTest::shouldParseVcardWithTwoOrg()
@@ -214,6 +230,7 @@ void OrgTest::shouldCreateVCardWithTwoOrg()
                         "END:VCARD\r\n\r\n");
 
     QCOMPARE(ba, expected);
+
 }
 
 void OrgTest::shouldCreateVCardWithParameters()
