@@ -69,9 +69,6 @@ public:
         mMailer = other.mMailer;
         mTimeZone = other.mTimeZone;
         mGeo = other.mGeo;
-        mTitle = other.mTitle;
-        mRole = other.mRole;
-        mOrganization = other.mOrganization;
         mDepartment = other.mDepartment;
         mNote = other.mNote;
         mProductId = other.mProductId;
@@ -101,6 +98,9 @@ public:
         mEmpty = other.mEmpty;
         mImpps = other.mImpps;
         mChanged = other.mChanged;
+        mTitleExtraList = other.mTitleExtraList;
+        mRoleExtraList = other.mRoleExtraList;
+        mOrgExtraList = other.mOrgExtraList;
     }
 
     ~Private()
@@ -120,9 +120,6 @@ public:
     QString mMailer;
     TimeZone mTimeZone;
     Geo mGeo;
-    QString mTitle;
-    QString mRole;
-    QString mOrganization;
     QString mDepartment;
     QString mNote;
     QString mProductId;
@@ -152,6 +149,9 @@ public:
     QStringList mMembers;
     Related::List mRelationShips;
     FieldGroup::List mFieldGroupList;
+    Title::List mTitleExtraList;
+    Role::List mRoleExtraList;
+    Org::List mOrgExtraList;
     bool mEmpty    : 1;
     bool mChanged  : 1;
 
@@ -259,24 +259,6 @@ bool Addressee::operator==(const Addressee &addressee) const
         return false;
     }
 
-    if (d->mTitle != addressee.d->mTitle &&
-            !(d->mTitle.isEmpty() && addressee.d->mTitle.isEmpty())) {
-        qCDebug(KCONTACTS_LOG) << "title differs";
-        return false;
-    }
-
-    if (d->mRole != addressee.d->mRole &&
-            !(d->mRole.isEmpty() && addressee.d->mRole.isEmpty())) {
-        qCDebug(KCONTACTS_LOG) << "role differs";
-        return false;
-    }
-
-    if (d->mOrganization != addressee.d->mOrganization &&
-            !(d->mOrganization.isEmpty() && addressee.d->mOrganization.isEmpty())) {
-        qCDebug(KCONTACTS_LOG) << "organization differs";
-        return false;
-    }
-
     if (d->mDepartment != addressee.d->mDepartment &&
             !(d->mDepartment.isEmpty() && addressee.d->mDepartment.isEmpty())) {
         qCDebug(KCONTACTS_LOG) << "department differs";
@@ -403,6 +385,20 @@ bool Addressee::operator==(const Addressee &addressee) const
         return false;
     }
 
+    if (!listEquals(d->mTitleExtraList, addressee.d->mTitleExtraList)) {
+        qCDebug(KCONTACTS_LOG) << "Extra TitleList differs";
+        return false;
+    }
+
+    if (!listEquals(d->mRoleExtraList, addressee.d->mRoleExtraList)) {
+        qCDebug(KCONTACTS_LOG) << "Extra RoleList differs";
+        return false;
+    }
+
+    if (!listEquals(d->mOrgExtraList, addressee.d->mOrgExtraList)) {
+        qCDebug(KCONTACTS_LOG) << "Extra Organization List differs";
+        return false;
+    }
     return true;
 }
 
@@ -949,17 +945,48 @@ QString Addressee::geoLabel()
 
 void Addressee::setTitle(const QString &title)
 {
-    if (title == d->mTitle) {
-        return;
+    Title t(title);
+    if (!d->mTitleExtraList.isEmpty()) {
+        t = d->mTitleExtraList.takeFirst();
+        t.setTitle(title);
+        d->mTitleExtraList.prepend(t);
+        d->mEmpty = false;
+    } else {
+        insertExtraTitle(title);
     }
+}
 
-    d->mEmpty = false;
-    d->mTitle = title;
+void Addressee::setTitle(const Title &title)
+{
+    insertExtraTitle(title);
+}
+
+void Addressee::insertExtraTitle(const Title &title)
+{
+    if (title.isValid()) {
+        d->mEmpty = false;
+        d->mTitleExtraList.append(title);
+    }
 }
 
 QString Addressee::title() const
 {
-    return d->mTitle;
+    if (d->mTitleExtraList.isEmpty()) {
+        return {};
+    } else {
+        return d->mTitleExtraList.at(0).title();
+    }
+}
+
+Title::List Addressee::extraTitleList() const
+{
+    return d->mTitleExtraList;
+}
+
+void Addressee::setExtraTitleList(const Title::List &urltitle)
+{
+    d->mEmpty = false;
+    d->mTitleExtraList = urltitle;
 }
 
 QString Addressee::titleLabel()
@@ -969,17 +996,49 @@ QString Addressee::titleLabel()
 
 void Addressee::setRole(const QString &role)
 {
-    if (role == d->mRole) {
-        return;
+    Role t(role);
+    if (!d->mRoleExtraList.isEmpty()) {
+        t = d->mRoleExtraList.takeFirst();
+        t.setRole(role);
+        d->mRoleExtraList.prepend(t);
+        d->mEmpty = false;
+    } else {
+        insertExtraRole(role);
     }
+}
 
+void Addressee::setRole(const Role &role)
+{
+    insertExtraRole(role);
+}
+
+void Addressee::insertExtraRole(const Role &role)
+{
+    if (role.isValid()) {
+        d->mEmpty = false;
+        d->mRoleExtraList.append(role);
+    }
+}
+
+void Addressee::setExtraRoleList(const Role::List &roleList)
+{
     d->mEmpty = false;
-    d->mRole = role;
+    d->mRoleExtraList = roleList;
+
+}
+
+Role::List Addressee::extraRoleList() const
+{
+    return d->mRoleExtraList;
 }
 
 QString Addressee::role() const
 {
-    return d->mRole;
+    if (d->mRoleExtraList.isEmpty()) {
+        return {};
+    } else {
+        return d->mRoleExtraList.at(0).role();
+    }
 }
 
 QString Addressee::roleLabel()
@@ -989,17 +1048,48 @@ QString Addressee::roleLabel()
 
 void Addressee::setOrganization(const QString &organization)
 {
-    if (organization == d->mOrganization) {
-        return;
+    Org t(organization);
+    if (!d->mOrgExtraList.isEmpty()) {
+        t = d->mOrgExtraList.takeFirst();
+        t.setOrganization(organization);
+        d->mOrgExtraList.prepend(t);
+        d->mEmpty = false;
+    } else {
+        insertExtraOrganization(organization);
     }
+}
 
+void Addressee::setOrganization(const Org &organization)
+{
+    insertExtraOrganization(organization);
+}
+
+void Addressee::insertExtraOrganization(const Org &organization)
+{
+    if (organization.isValid()) {
+        d->mEmpty = false;
+        d->mOrgExtraList.append(organization);
+    }
+}
+
+void Addressee::setExtraOrganizationList(const Org::List &orgList)
+{
     d->mEmpty = false;
-    d->mOrganization = organization;
+    d->mOrgExtraList = orgList;
+}
+
+Org::List Addressee::extraOrganizationList() const
+{
+    return d->mOrgExtraList;
 }
 
 QString Addressee::organization() const
 {
-    return d->mOrganization;
+    if (d->mOrgExtraList.isEmpty()) {
+        return {};
+    } else {
+        return d->mOrgExtraList.at(0).organization();
+    }
 }
 
 QString Addressee::organizationLabel()
@@ -1343,7 +1433,7 @@ void Addressee::setNameFromString(const QString &s)
         leftOffset = 0;
         rightOffset = parts.count();
 
-        if (parts.count() > 0) {
+        if (!parts.isEmpty()) {
 
             QString prefix;
             while (leftOffset < rightOffset) {
@@ -1571,7 +1661,8 @@ void Addressee::insertPhoneNumber(const PhoneNumber &phoneNumber)
     d->mEmpty = false;
 
     PhoneNumber::List::Iterator it;
-    for (it = d->mPhoneNumbers.begin(); it != d->mPhoneNumbers.end(); ++it) {
+    PhoneNumber::List::Iterator end(d->mPhoneNumbers.end());
+    for (it = d->mPhoneNumbers.begin(); it != end; ++it) {
         if ((*it).id() == phoneNumber.id()) {
             *it = phoneNumber;
             return;
@@ -1837,7 +1928,8 @@ void Addressee::insertAddress(const Address &address)
     d->mEmpty = false;
 
     Address::List::Iterator it;
-    for (it = d->mAddresses.begin(); it != d->mAddresses.end(); ++it) {
+    Address::List::Iterator end(d->mAddresses.end());
+    for (it = d->mAddresses.begin(); it != end; ++it) {
         if ((*it).id() == address.id()) {
             *it = address;
             return;
@@ -2272,9 +2364,6 @@ QDataStream &KContacts::operator<<(QDataStream &s, const Addressee &a)
     s << a.d->mMailer;
     s << a.d->mTimeZone;
     s << a.d->mGeo;
-    s << a.d->mTitle;
-    s << a.d->mRole;
-    s << a.d->mOrganization;
     s << a.d->mDepartment;
     s << a.d->mNote;
     s << a.d->mProductId;
@@ -2303,6 +2392,9 @@ QDataStream &KContacts::operator<<(QDataStream &s, const Addressee &a)
     s << a.d->mSources;
     s << a.d->mImpps;
     s << a.d->mFieldGroupList;
+    s << a.d->mTitleExtraList;
+    s << a.d->mRoleExtraList;
+    s << a.d->mOrgExtraList;
 
     return s;
 }
@@ -2323,9 +2415,6 @@ QDataStream &KContacts::operator>>(QDataStream &s, Addressee &a)
     s >> a.d->mMailer;
     s >> a.d->mTimeZone;
     s >> a.d->mGeo;
-    s >> a.d->mTitle;
-    s >> a.d->mRole;
-    s >> a.d->mOrganization;
     s >> a.d->mDepartment;
     s >> a.d->mNote;
     s >> a.d->mProductId;
@@ -2356,6 +2445,9 @@ QDataStream &KContacts::operator>>(QDataStream &s, Addressee &a)
     s >> a.d->mSources;
     s >> a.d->mImpps;
     s >> a.d->mFieldGroupList;
+    s >> a.d->mTitleExtraList;
+    s >> a.d->mRoleExtraList;
+    s >> a.d->mOrgExtraList;
     a.d->mEmpty = false;
 
     return s;
