@@ -45,6 +45,10 @@
 #include "addresseelist.h"
 #include "resourcelocatorurl.h"
 #include "fieldgroup.h"
+#include "title.h"
+#include "nickname.h"
+#include "role.h"
+#include "org.h"
 
 namespace KContacts
 {
@@ -452,8 +456,12 @@ public:
     /**
       Set title.
      */
+    //Remove in kf6
     void setTitle(const QString &title);
-
+    void setTitle(const Title &title);
+    void insertExtraTitle(const Title &title);
+    void setExtraTitleList(const Title::List &urltitle);
+    Title::List extraTitleList() const;
     /**
       Return title.
      */
@@ -468,6 +476,10 @@ public:
       Set role.
      */
     void setRole(const QString &role);
+    void setRole(const Role &role);
+    void insertExtraRole(const Role &role);
+    void setExtraRoleList(const Role::List &roleList);
+    Role::List extraRoleList() const;
 
     /**
       Return role.
@@ -482,7 +494,13 @@ public:
     /**
       Set organization.
      */
+    //Remove in kf6
     void setOrganization(const QString &organization);
+    void setOrganization(const Org &organization);
+    void insertExtraOrganization(const Org &organization);
+    void setExtraOrganizationList(const Org::List &orgList);
+    Org::List extraOrganizationList() const;
+
 
     /**
       Return organization.
