@@ -88,6 +88,15 @@ QByteArray VCardTool::createVCards(const Addressee::List &list, VCard::Version v
     return createVCards(list, version, false /*don't export*/);
 }
 
+void VCardTool::addParameters(VCardLine &line, const QMap<QString, QStringList> &params) const
+{
+    QMapIterator<QString, QStringList> i(params);
+    while (i.hasNext()) {
+        i.next();
+        line.addParameter(i.key(), i.value().join(QLatin1Char(',')));
+    }
+}
+
 void VCardTool::addParameter(VCardLine &line, VCard::Version version, const QString &key, const QStringList &valueStringList) const
 {
     if (version == VCard::v2_1) {
@@ -117,16 +126,17 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         QStringList::ConstIterator strIt;
         // VERSION
         if (version == VCard::v2_1) {
-            card.addLine(VCardLine(QStringLiteral("VERSION"), QLatin1String("2.1")));
+            card.addLine(VCardLine(QStringLiteral("VERSION"), QStringLiteral("2.1")));
         } else if (version == VCard::v3_0) {
-            card.addLine(VCardLine(QStringLiteral("VERSION"), QLatin1String("3.0")));
+            card.addLine(VCardLine(QStringLiteral("VERSION"), QStringLiteral("3.0")));
         } else if (version == VCard::v4_0) {
-            card.addLine(VCardLine(QStringLiteral("VERSION"), QLatin1String("4.0")));
+            card.addLine(VCardLine(QStringLiteral("VERSION"), QStringLiteral("4.0")));
         }
 
         // ADR + LABEL
         const Address::List addresses = (*addrIt).addresses();
-        for (Address::List::ConstIterator it = addresses.begin(); it != addresses.end(); ++it) {
+        Address::List::ConstIterator end(addresses.end());
+        for (Address::List::ConstIterator it = addresses.begin(); it != end; ++it) {
             QStringList address;
 
             const bool isEmpty = ((*it).postOfficeBox().isEmpty() &&
@@ -138,33 +148,33 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
                                   (*it).country().isEmpty());
 
             address.append((*it).postOfficeBox().replace(QLatin1Char(';'),
-                           QLatin1String("\\;")));
+                           QStringLiteral("\\;")));
 
             address.append((*it).extended().replace(QLatin1Char(';'),
-                                                    QLatin1String("\\;")));
+                                                    QStringLiteral("\\;")));
 
             address.append((*it).street().replace(QLatin1Char(';'),
-                                                  QLatin1String("\\;")));
+                                                  QStringLiteral("\\;")));
 
             address.append((*it).locality().replace(QLatin1Char(';'),
-                                                    QLatin1String("\\;")));
+                                                    QStringLiteral("\\;")));
 
             address.append((*it).region().replace(QLatin1Char(';'),
-                                                  QLatin1String("\\;")));
+                                                  QStringLiteral("\\;")));
 
             address.append((*it).postalCode().replace(QLatin1Char(';'),
-                           QLatin1String("\\;")));
+                           QStringLiteral("\\;")));
 
             address.append((*it).country().replace(QLatin1Char(';'),
-                                                   QLatin1String("\\;")));
+                                                   QStringLiteral("\\;")));
 
-            VCardLine adrLine(QStringLiteral("ADR"), address.join(QLatin1Char(';')));
-            if (version == VCard::v2_1 && needsEncoding(address.join(QLatin1Char(';')))) {
+            const QString addressJoined(address.join(QLatin1Char(';')));
+            VCardLine adrLine(QStringLiteral("ADR"), addressJoined);
+            if (version == VCard::v2_1 && needsEncoding(addressJoined)) {
                 adrLine.addParameter(QStringLiteral("charset"), QStringLiteral("UTF-8"));
                 adrLine.addParameter(QStringLiteral("encoding"), QStringLiteral("QUOTED-PRINTABLE"));
             }
 
-
             const bool hasLabel = !(*it).label().isEmpty();
             QMap<QString, Address::TypeFlag>::ConstIterator typeIt;
             QStringList addreLineType;
@@ -220,7 +230,7 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
                 QStringList::Iterator catIt;
                 QStringList::Iterator catEnd(categories.end());
                 for (catIt = categories.begin(); catIt != catEnd; ++catIt) {
-                    (*catIt).replace(QLatin1Char(','), QLatin1String("\\,"));
+                    (*catIt).replace(QLatin1Char(','), QStringLiteral("\\,"));
                 }
 
                 VCardLine catLine(QStringLiteral("CATEGORIES"), categories.join(QLatin1Char(',')));
@@ -248,11 +258,7 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         Related::List::ConstIterator relatedEnd(relatedList.end());
         for (relatedIt = relatedList.begin(); relatedIt != relatedEnd; ++relatedIt) {
             VCardLine line(QStringLiteral("RELATED"), (*relatedIt).related());
-            QMapIterator<QString, QStringList> i((*relatedIt).parameters());
-            while (i.hasNext()) {
-                i.next();
-                line.addParameter(i.key(), i.value().join(QLatin1Char(',')));
-            }
+            addParameters(line, (*relatedIt).parameters());
             card.addLine(line);
         }
         // CLASS only for version == 3.0
@@ -266,11 +272,7 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
             Lang::List::ConstIterator langEnd(langList.end());
             for (langIt = langList.begin(); langIt != langEnd; ++langIt) {
                 VCardLine line(QStringLiteral("LANG"), (*langIt).language());
-                QMapIterator<QString, QStringList> i((*langIt).parameters());
-                while (i.hasNext()) {
-                    i.next();
-                    line.addParameter(i.key(), i.value().join(QLatin1Char(',')));
-                }
+                addParameters(line, (*langIt).parameters());
                 card.addLine(line);
             }
         }
@@ -354,19 +356,19 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         // N required for only version < 4.0
         QStringList name;
         name.append((*addrIt).familyName().replace(QLatin1Char(';'),
-                    QLatin1String("\\;")));
+                    QStringLiteral("\\;")));
 
         name.append((*addrIt).givenName().replace(QLatin1Char(';'),
-                    QLatin1String("\\;")));
+                    QStringLiteral("\\;")));
 
         name.append((*addrIt).additionalName().replace(QLatin1Char(';'),
-                    QLatin1String("\\;")));
+                    QStringLiteral("\\;")));
 
         name.append((*addrIt).prefix().replace(QLatin1Char(';'),
-                                               QLatin1String("\\;")));
+                                               QStringLiteral("\\;")));
 
         name.append((*addrIt).suffix().replace(QLatin1Char(';'),
-                                               QLatin1String("\\;")));
+                                               QStringLiteral("\\;")));
 
         VCardLine nLine(QStringLiteral("N"), name.join(QLatin1Char(';')));
         if (version == VCard::v2_1 && needsEncoding(name.join(QLatin1Char(';')))) {
@@ -403,19 +405,23 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         card.addLine(noteLine);
 
         // ORG
-        QStringList organization;
-        organization.append((*addrIt).organization().replace(QLatin1Char(';'),
-                            QLatin1String("\\;")));
-        if (!(*addrIt).department().isEmpty()) {
-            organization.append((*addrIt).department().replace(QLatin1Char(';'),
-                                QLatin1String("\\;")));
+        Q_FOREACH (const Org &org, (*addrIt).extraOrganizationList()) {
+            QStringList organization;
+            organization.append(org.organization().replace(QLatin1Char(';'),
+                                QStringLiteral("\\;")));
+            if (!(*addrIt).department().isEmpty()) {
+                organization.append((*addrIt).department().replace(QLatin1Char(';'),
+                                    QStringLiteral("\\;")));
+            }
+            const QString orgStr = organization.join(QLatin1Char(';'));
+            VCardLine orgLine(QStringLiteral("ORG"), orgStr);
+            if (version == VCard::v2_1 && needsEncoding(orgStr)) {
+                orgLine.addParameter(QStringLiteral("charset"), QStringLiteral("UTF-8"));
+                orgLine.addParameter(QStringLiteral("encoding"), QStringLiteral("QUOTED-PRINTABLE"));
+            }
+            addParameters(orgLine, org.parameters());
+            card.addLine(orgLine);
         }
-        VCardLine orgLine(QStringLiteral("ORG"), organization.join(QLatin1Char(';')));
-        if (version == VCard::v2_1 && needsEncoding(organization.join(QLatin1Char(';')))) {
-            orgLine.addParameter(QStringLiteral("charset"), QStringLiteral("UTF-8"));
-            orgLine.addParameter(QStringLiteral("encoding"), QStringLiteral("QUOTED-PRINTABLE"));
-        }
-        card.addLine(orgLine);
 
         // PHOTO
         card.addLine(createPicture(QStringLiteral("PHOTO"), (*addrIt).photo(), version));
@@ -432,12 +438,15 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         card.addLine(VCardLine(QStringLiteral("REV"), createDateTime((*addrIt).revision(), version)));
 
         // ROLE
-        VCardLine roleLine(QStringLiteral("ROLE"), (*addrIt).role());
-        if (version == VCard::v2_1 && needsEncoding((*addrIt).role())) {
-            roleLine.addParameter(QStringLiteral("charset"), QStringLiteral("UTF-8"));
-            roleLine.addParameter(QStringLiteral("encoding"), QStringLiteral("QUOTED-PRINTABLE"));
+        Q_FOREACH (const Role &role, (*addrIt).extraRoleList()) {
+            VCardLine roleLine(QStringLiteral("ROLE"), role.role());
+            if (version == VCard::v2_1 && needsEncoding(role.role())) {
+                roleLine.addParameter(QStringLiteral("charset"), QStringLiteral("UTF-8"));
+                roleLine.addParameter(QStringLiteral("encoding"), QStringLiteral("QUOTED-PRINTABLE"));
+            }
+            addParameters(roleLine, role.parameters());
+            card.addLine(roleLine);
         }
-        card.addLine(roleLine);
 
         // SORT-STRING
         if (version == VCard::v3_0) {
@@ -456,6 +465,13 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         PhoneNumber::List::ConstIterator phoneEnd(phoneNumbers.end());
         for (phoneIt = phoneNumbers.begin(); phoneIt != phoneEnd; ++phoneIt) {
             VCardLine line(QStringLiteral("TEL"), (*phoneIt).number());
+            QMapIterator<QString, QStringList> i((*phoneIt).parameters());
+            while (i.hasNext()) {
+                i.next();
+                if (i.key().toUpper() != QLatin1String("TYPE")) {
+                    line.addParameter(i.key(), i.value().join(QLatin1Char(',')));
+                }
+            }
 
             QMap<QString, PhoneNumber::TypeFlag>::ConstIterator typeIt;
             QMap<QString, PhoneNumber::TypeFlag>::ConstIterator typeEnd(mPhoneTypeMap.constEnd());
@@ -476,12 +492,16 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         }
 
         // TITLE
-        VCardLine titleLine(QStringLiteral("TITLE"), (*addrIt).title());
-        if (version == VCard::v2_1 && needsEncoding((*addrIt).title())) {
-            titleLine.addParameter(QStringLiteral("charset"), QStringLiteral("UTF-8"));
-            titleLine.addParameter(QStringLiteral("encoding"), QStringLiteral("QUOTED-PRINTABLE"));
+        Q_FOREACH (const Title &title, (*addrIt).extraTitleList()) {
+            VCardLine titleLine(QStringLiteral("TITLE"), title.title());
+            if (version == VCard::v2_1 && needsEncoding(title.title())) {
+                titleLine.addParameter(QStringLiteral("charset"), QStringLiteral("UTF-8"));
+                titleLine.addParameter(QStringLiteral("encoding"), QStringLiteral("QUOTED-PRINTABLE"));
+            }
+            addParameters(titleLine, title.parameters());
+
+            card.addLine(titleLine);
         }
-        card.addLine(titleLine);
 
         // TZ
         // TODO Add vcard4.0 support
@@ -507,11 +527,7 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         // URL
         Q_FOREACH (const ResourceLocatorUrl &url, (*addrIt).extraUrlList()) {
             VCardLine line(QStringLiteral("URL"), url.url());
-            QMapIterator<QString, QStringList> i(url.parameters());
-            while (i.hasNext()) {
-                i.next();
-                line.addParameter(i.key(), i.value().join(QLatin1Char(',')));
-            }
+            addParameters(line, url.parameters());
             card.addLine(line);
         }
         if (version == VCard::v4_0) {
@@ -556,11 +572,7 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
                     }
                     if (!type.isEmpty()) {
                         VCardLine line(type, url.url().toDisplayString());
-                        QMapIterator<QString, QStringList> i(url.parameters());
-                        while (i.hasNext()) {
-                            i.next();
-                            line.addParameter(i.key(), i.value().join(QLatin1Char(',')));
-                        }
+                        addParameters(line, url.parameters());
                         card.addLine(line);
                     }
                 }
@@ -569,11 +581,7 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         //FieldGroup
         Q_FOREACH (const FieldGroup &group, (*addrIt).fieldGroupList()) {
             VCardLine line(group.fieldGroupName(), group.value());
-            QMapIterator<QString, QStringList> i(group.parameters());
-            while (i.hasNext()) {
-                i.next();
-                line.addParameter(i.key(), i.value().join(QLatin1Char(',')));
-            }
+            addParameters(line, group.parameters());
             card.addLine(line);
         }
 
@@ -699,25 +707,26 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                 else if (identifier == QLatin1String("adr")) {
                     Address address;
                     const QStringList addrParts = splitString(semicolonSep, (*lineIt).value().toString());
-                    if (addrParts.count() > 0) {
+                    const int addrPartsCount(addrParts.count());
+                    if (addrPartsCount > 0) {
                         address.setPostOfficeBox(addrParts.at(0));
                     }
-                    if (addrParts.count() > 1) {
+                    if (addrPartsCount > 1) {
                         address.setExtended(addrParts.at(1));
                     }
-                    if (addrParts.count() > 2) {
+                    if (addrPartsCount > 2) {
                         address.setStreet(addrParts.at(2));
                     }
-                    if (addrParts.count() > 3) {
+                    if (addrPartsCount > 3) {
                         address.setLocality(addrParts.at(3));
                     }
-                    if (addrParts.count() > 4) {
+                    if (addrPartsCount > 4) {
                         address.setRegion(addrParts.at(4));
                     }
-                    if (addrParts.count() > 5) {
+                    if (addrPartsCount > 5) {
                         address.setPostalCode(addrParts.at(5));
                     }
-                    if (addrParts.count() > 6) {
+                    if (addrPartsCount > 6) {
                         address.setCountry(addrParts.at(6));
                     }
 
@@ -734,7 +743,7 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                     if (!label.isEmpty()) {
                         if (label.length() > 1) {
                             if (label.at(0) == QLatin1Char('"') && label.at(label.length() - 1) == QLatin1Char('"')) {
-                                label = label.mid(1, label.length() - 2 );
+                                label = label.mid(1, label.length() - 2);
                             }
                         }
                         address.setLabel(label);
@@ -975,7 +984,9 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                     const QStringList orgParts = splitString(semicolonSep, (*lineIt).value().toString());
                     const int orgPartsCount(orgParts.count());
                     if (orgPartsCount > 0) {
-                        addr.setOrganization(orgParts.at(0));
+                        Org organization(orgParts.at(0));
+                        organization.setParameters((*lineIt).parameterMap());
+                        addr.insertExtraOrganization(organization);
                     }
                     if (orgPartsCount > 1) {
                         addr.setDepartment(orgParts.at(1));
@@ -1007,7 +1018,9 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
 
                 // ROLE
                 else if (identifier == QLatin1String("role")) {
-                    addr.setRole((*lineIt).value().toString());
+                    Role role((*lineIt).value().toString());
+                    role.setParameters((*lineIt).parameterMap());
+                    addr.insertExtraRole(role);
                 }
 
                 // SORT-STRING
@@ -1027,16 +1040,11 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
 
                 // TEL
                 else if (identifier == QLatin1String("tel")) {
-                    //TODO implement load vcard4
                     PhoneNumber phone;
                     phone.setNumber((*lineIt).value().toString());
 
                     PhoneNumber::Type type;
                     bool foundType = false;
-#if 0
-                    qDebug() << " (*lineIt).parameters" << (*lineIt).parameterMap();
-                    qDebug() << " (*lineIt).value()" << (*lineIt).value().toString();
-#endif
                     const QStringList types = (*lineIt).parameters(QStringLiteral("type"));
                     QStringList::ConstIterator typeEnd(types.constEnd());
                     for (QStringList::ConstIterator it = types.constBegin(); it != typeEnd; ++it) {
@@ -1044,13 +1052,16 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                         foundType = true;
                     }
                     phone.setType(foundType ? type : PhoneNumber::Undefined);
+                    phone.setParameters((*lineIt).parameterMap());
 
                     addr.insertPhoneNumber(phone);
                 }
 
                 // TITLE
                 else if (identifier == QLatin1String("title")) {
-                    addr.setTitle((*lineIt).value().toString());
+                    Title title((*lineIt).value().toString());
+                    title.setParameters((*lineIt).parameterMap());
+                    addr.insertExtraTitle(title);
                 }
 
                 // TZ
@@ -1436,11 +1447,11 @@ VCardLine VCardTool::createSecrecy(const Secrecy &secrecy) const
     int type = secrecy.type();
 
     if (type == Secrecy::Public) {
-        line.setValue(QLatin1String("PUBLIC"));
+        line.setValue(QStringLiteral("PUBLIC"));
     } else if (type == Secrecy::Private) {
-        line.setValue(QLatin1String("PRIVATE"));
+        line.setValue(QStringLiteral("PRIVATE"));
     } else if (type == Secrecy::Confidential) {
-        line.setValue(QLatin1String("CONFIDENTIAL"));
+        line.setValue(QStringLiteral("CONFIDENTIAL"));
     }
 
     return line;
@@ -1481,7 +1492,6 @@ QStringList VCardTool::splitString(QChar sep, const QString &str) const
     return list;
 }
 
-
 void VCardTool::imppService(const QString &serviceType, KContacts::Impp &impp) const
 {
     if (serviceType == QLatin1String("facebook")) {
